'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.FieldStatus = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; // @ts-check
/* eslint-disable no-param-reassign */
// This file contain any `private` method for the Veasy

exports.getConstructorErrorMessage = getConstructorErrorMessage;
exports.typeCheck = typeCheck;
exports.createInitialValue = createInitialValue;
exports.createNewFieldState = createNewFieldState;
exports.createInitialState = createInitialState;
exports.shouldChange = shouldChange;
exports.getFieldsValue = getFieldsValue;
exports.throwError = throwError;
exports.resetForm = resetForm;
exports.rulesRunner = rulesRunner;
exports.checkIsFormOK = checkIsFormOK;
exports.startValidating = startValidating;
exports.validate = validate;

var _is_js = require('is_js');

var _is_js2 = _interopRequireDefault(_is_js);

var _matchers = require('./ruleHandlers/matchers');

var _matchers2 = _interopRequireDefault(_matchers);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var FieldStatus = exports.FieldStatus = {
  ok: 'ok',
  error: 'error',
  normal: 'normal'
};

/**
 * Return error message for checking the parameters of the constructor.
 */
function getConstructorErrorMessage(paramName, value) {
  return '[Veasy - ' + paramName + '] Expect: non empty object. Actual: ' + value;
}

/**
 * Check if an object is a non-empty object
 */
function isNonEmptyObject(obj) {
  return _is_js2.default.object(obj) && _is_js2.default.not.empty(obj);
}

/**
 * Type check for 2 parameters of the constructor
 *
 */
function typeCheck(component, schema) {
  var isComponentValid = isNonEmptyObject(component);
  var isSchemaValid = isNonEmptyObject(schema);

  if (!isComponentValid) {
    throw new Error(getConstructorErrorMessage('Parameter component', component));
  }

  if (!isSchemaValid) {
    throw new Error(getConstructorErrorMessage('Parameter schema', schema));
  }

  Object.keys(schema).forEach(function (prop) {
    if (!isNonEmptyObject(schema[prop])) {
      throw new Error(getConstructorErrorMessage('schema.' + prop, schema[prop]));
    }
  });
}

/**
 * Create initial value for a field if no default is provided.
 *
 */
function createInitialValue(schema) {
  if (_is_js2.default.propertyDefined(schema, 'default')) {
    return schema.default;
  } else if (_is_js2.default.propertyDefined(schema, 'min')) {
    return schema.min;
  }
  return '';
}

/**
 * Create a new state for a field in componentState.formStatus.fields.field
 *
 */
function createNewFieldState() {
  var needValue = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : false;
  var fieldSchema = arguments[1];

  var result = {
    status: FieldStatus.normal,
    errorText: ''
  };
  if (needValue) {
    result.value = createInitialValue(fieldSchema);
  }
  return result;
}

function createInitialState(schema, userState) {
  var initialState = _extends({
    isFormOK: false
  }, userState);

  Object.keys(schema).forEach(function (prop) {
    if (prop === 'collectValues') return;
    initialState[prop] = createNewFieldState(true, schema[prop]);
  });

  var schemaItems = Object.keys(schema);
  schemaItems.forEach(function (name) {
    if (_is_js2.default.propertyDefined(userState, name)) {
      initialState[name] = _extends({}, initialState[name], userState[name]);
    }
  });

  return initialState;
}

/**
 * Check if we should change the state or not.
 *
 */
function shouldChange(oldState, newState) {
  var isErrorDifferent = oldState.status !== newState.status;
  var isValueDifferent = oldState.value !== newState.value;
  return isErrorDifferent || isValueDifferent;
}

function getNestedValue(key, obj) {
  return key.split('.').reduce(function (result1, key1) {
    return result1[key1];
  }, obj);
}

function getCollectValues(collectSchema, state) {
  var fieldsToCollect = Object.keys(collectSchema);
  var result = {};

  fieldsToCollect.forEach(function (fieldName) {
    result[fieldName] = getNestedValue(collectSchema[fieldName], state);
  });

  return result;
}

function getFieldsValue(schema, state) {
  var mustOK = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : true;

  var fieldNames = Object.keys(schema);
  var result = {};

  if (_is_js2.default.propertyDefined(schema, 'collectValues')) {
    result = _extends({}, getCollectValues(schema.collectValues, state));
  }

  fieldNames.forEach(function (name) {
    if (_is_js2.default.not.propertyDefined(state, name)) {
      // eslint-disable-next-line no-console
      console.warn('[veasy]: No ' + name + ' found in state.');
      return;
    }
    var fieldState = state[name];
    if (mustOK && fieldState.status !== FieldStatus.ok) return;
    result[name] = fieldState.value;
  });

  return result;
}

/**
 * throw an error with defined text, usually calls by ruleRunner().
 */
function throwError(value, errorText) {
  var error = { value: value, errorText: errorText, status: FieldStatus.error };
  throw error;
}

function extractUserDefinedMsg(handlerName, schema) {
  var result = { schema: schema, userErrorText: '' };

  // No user message, just return
  if (_is_js2.default.not.array(schema[handlerName])) return result;

  var currentSchema = schema[handlerName];

  // Handle the case where the value of rule is array
  if (_matchers.RuleWhichNeedsArray.includes(handlerName)) {
    // No user message, just return
    if (_is_js2.default.not.array(currentSchema[0])) return result;
  }

  // The most common case: [0] is rule and [1] is errText
  result.schema = _defineProperty({}, handlerName, currentSchema[0]);
  // eslint-disable-next-line prefer-destructuring
  result.userErrorText = currentSchema[1];
  return result;
}

function ruleRunner(ruleHandler, fieldName, value, pschema) {
  var _extractUserDefinedMs = extractUserDefinedMsg(ruleHandler.name, pschema),
      schema = _extractUserDefinedMs.schema,
      userErrorText = _extractUserDefinedMs.userErrorText;

  var ruleName = ruleHandler.name;

  if (_matchers.RuleWhichNeedsBoolean.includes(ruleName)) {
    if (schema[ruleName] === false) return;
  }

  var result = ruleHandler(fieldName, value, schema);
  if (result.isValid) return;

  throwError(value, userErrorText || result.errorText);
}

function resetForm(schema, state) {
  var newSchema = _extends({}, schema);
  delete newSchema.collectValues;
  var newState = _extends({}, state);
  var fieldNames = Object.keys(newSchema);
  fieldNames.forEach(function (name) {
    var newField = newState[name];
    newField.status = FieldStatus.normal;
    newField.errorText = '';
    newField.value = createInitialValue(schema[name]);
  });
  newState.isFormOK = false;
  return newState;
}

/**
 * It will run through the user's settings for a field,
 * and try matching to the matchers.js,
 * if according rule could be found,
 * it will then execute the according rule function.
 * For instance:
 * if user sets a `minLength` for a field,
 * This function will invoke the minLength()
 *
 */
function runMatchers(matcher, fieldState, fieldSchema) {
  var fieldName = Object.keys(fieldSchema)[0];
  var schema = fieldSchema[fieldName];
  Object.keys(schema).forEach(function (ruleInSchema) {
    if (_is_js2.default.propertyDefined(matcher, ruleInSchema)) {
      ruleRunner(matcher[ruleInSchema], fieldName, fieldState.value, schema);
    }
    // TODO: Do something when the rule is not match
    // else if (ruleInSchema !== 'default') {
    // }
  });
  return fieldState;
}

/**
 * This is the main entry for all validator.
 * It will generate the initial state to start with
 *
 */
function rulesRunner(value, schema) {
  var fieldState = createNewFieldState();
  fieldState.value = value;

  if (_is_js2.default.existy(value) && _is_js2.default.not.empty(value)) {
    fieldState.status = FieldStatus.ok;
  }

  return runMatchers(_matchers2.default, fieldState, schema);
}

/**
 * If all fields in the state has their status !== error
 * Then we will set the isFormOK to true then return the state.
 * Just mutate the value since it's already a new state object
 *
 */
function checkIsFormOK(schema, componentState) {
  var properties = Object.keys(schema);
  var isError = false;
  properties.some(function (prop) {
    if (prop === 'collectValues') return false;
    if (_is_js2.default.propertyDefined(schema[prop], 'isRequired') && schema[prop].isRequired === false && componentState[prop].status !== FieldStatus.error) return false;

    if (componentState[prop].status === FieldStatus.error || componentState[prop].status === FieldStatus.normal) {
      isError = true;
      return true;
    }
    return false;
  });
  if (!isError) {
    componentState.isFormOK = true;
  } else {
    componentState.isFormOK = false;
  }

  return componentState;
}

function updateWhenNeeded(newFieldState, propName, update, schema) {
  var formState = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : undefined;

  var fieldState = _defineProperty({}, propName, newFieldState);
  if (formState === undefined) {
    update(fieldState);
    return;
  }

  var oldFieldState = formState[propName];
  var newFieldState1 = _extends({}, oldFieldState, fieldState[propName]);

  if (_is_js2.default.existy(oldFieldState) && _is_js2.default.existy(newFieldState)) {
    if (!shouldChange(oldFieldState, newFieldState)) return;
  } else {
    return;
  }

  var finalState = _extends({}, formState, _defineProperty({}, propName, _extends({}, newFieldState1)));
  update(checkIsFormOK(schema, finalState));
}

function startValidating(target, schema, update, allState) {
  var targetName = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : undefined;

  var propName = targetName || target.name;

  if (_is_js2.default.not.existy(propName)) {
    throw new Error('target.name and targetName are both non-existy');
  }

  var fieldInfo = {
    value: target.value,
    schema: _defineProperty({}, propName, schema[propName])
  };

  return Promise.resolve(fieldInfo)
  // eslint-disable-next-line arrow-body-style
  .then(function (info) {
    return rulesRunner(info.value, info.schema);
  }).catch(function (errorState) {
    return errorState;
  }).then(function (newFieldState) {
    return updateWhenNeeded(newFieldState, propName, update, schema, allState);
  });
}

function validate(e, schema, allState, update, targetName) {
  e.persist();
  startValidating(e.target, schema, update, allState, targetName);
}
//# sourceMappingURL=helpers.js.map