'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.isIP = exports.notEmpty = exports.isHexColor = exports.isCreditCard = exports.isUrl = exports.isEmail = exports.matchRegex = exports.inArray = undefined;

var _is_js = require('is_js');

var _is_js2 = _interopRequireDefault(_is_js);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var inArray = exports.inArray = function inArray(name, value, schema) {
  return {
    isValid: _is_js2.default.inArray(value, schema.inArray),
    errorText: 'Value of ' + name + ' should be within [' + schema.inArray.toString() + '].'
  };
}; // @ts-check

var matchRegex = exports.matchRegex = function matchRegex(name, value, schema) {
  return {
    isValid: schema.matchRegex.test(value),
    errorText: 'Value of ' + name + ' is not valid.'
  };
};

var isEmail = exports.isEmail = function isEmail(name, value) {
  return {
    isValid: _is_js2.default.email(value),
    errorText: name + ' should be email.'
  };
};

var isUrl = exports.isUrl = function isUrl(name, value) {
  return {
    isValid: _is_js2.default.url(value),
    errorText: name + ' should be a URL.'
  };
};

var isCreditCard = exports.isCreditCard = function isCreditCard(name, value) {
  return {
    isValid: _is_js2.default.creditCard(value * 1),
    errorText: name + ' should be a credit card number.'
  };
};

var isHexColor = exports.isHexColor = function isHexColor(name, value) {
  return {
    isValid: _is_js2.default.hexColor(value),
    errorText: name + ' should be a hex color.'
  };
};

var notEmpty = exports.notEmpty = function notEmpty(name, value) {
  return {
    isValid: _is_js2.default.not.empty(value),
    errorText: name + ' should not be empty.'
  };
};

var isIP = exports.isIP = function isIP(name, value, schema) {
  var isValid = void 0;
  var ipString = '';

  switch (schema.isIP) {
    case 'v4':
      isValid = _is_js2.default.ipv4(value);
      ipString = 'IPv4';
      break;
    case 'v6':
      isValid = _is_js2.default.ipv6(value);
      ipString = 'IPv6';
      break;
    case 'all':
      isValid = _is_js2.default.ip(value);
      ipString = 'IP';
      break;
    default:
      isValid = _is_js2.default.ip(value);
      ipString = 'IP';
      break;
  }

  return {
    isValid: isValid,
    errorText: name + ' should be ' + ipString + ' address.'
  };
};
//# sourceMappingURL=generalRules.js.map