'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.isIntOrDecimal = exports.isDecimal = exports.isInt = exports.isNegative = exports.isPositive = exports.notEqual = exports.equal = exports.max = exports.min = undefined;

var _is_js = require('is_js');

var _is_js2 = _interopRequireDefault(_is_js);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var min = exports.min = function min(name, value, schema) {
  return {
    isValid: _is_js2.default.above(value * 1, schema.min * 1),
    errorText: name + ' should be greater than ' + schema.min + '. Current: ' + value
  };
};

var max = exports.max = function max(name, value, schema) {
  return {
    isValid: _is_js2.default.under(value * 1, schema.max * 1),
    errorText: name + ' should be less than ' + schema.max + '. Current: ' + value
  };
};

var equal = exports.equal = function equal(name, value, schema) {
  return {
    isValid: _is_js2.default.equal(value * 1, schema.equal * 1),
    errorText: name + ' should equal to ' + schema.equal * 1 + '.'
  };
};

var notEqual = exports.notEqual = function notEqual(name, value, schema) {
  return {
    isValid: _is_js2.default.not.equal(value * 1, schema.notEqual * 1),
    errorText: name + ' should not equal to ' + schema.notEqual * 1 + '.'
  };
};

var isPositive = exports.isPositive = function isPositive(name, value) {
  return {
    isValid: _is_js2.default.positive(value * 1),
    errorText: name + ' should be positive.'
  };
};

var isNegative = exports.isNegative = function isNegative(name, value) {
  return {
    isValid: _is_js2.default.negative(value * 1),
    errorText: name + ' should be negative.'
  };
};

var isInt = exports.isInt = function isInt(name, value) {
  return {
    isValid: _is_js2.default.integer(value * 1),
    errorText: name + ' should be integer.'
  };
};

var isDecimal = exports.isDecimal = function isDecimal(name, value) {
  return {
    isValid: _is_js2.default.decimal(value * 1),
    errorText: name + ' should be decimal.'
  };
};

var isIntOrDecimal = exports.isIntOrDecimal = function isIntOrDecimal(name, value) {
  return {
    isValid: _is_js2.default.integer(value * 1) || _is_js2.default.decimal(value * 1),
    errorText: name + ' should be either integer or decimal.'
  };
};
//# sourceMappingURL=numberRules.js.map